package eu.jonahbauer.wizard.client.cli.state;

import eu.jonahbauer.wizard.client.cli.Client;
import eu.jonahbauer.wizard.common.messages.server.AckMessage;
import eu.jonahbauer.wizard.common.messages.server.Response;
import eu.jonahbauer.wizard.common.messages.server.ServerMessage;

import java.util.Optional;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

public final class AwaitingAcknowledgement extends Awaiting {
    private final Supplier<ClientState> success;
    private final Function<ServerMessage, ClientState> failure;

    public AwaitingAcknowledgement(Supplier<ClientState> success, Function<ServerMessage, ClientState> failure) {
        this.success = success;
        this.failure = failure;
    }

    public AwaitingAcknowledgement(Supplier<ClientState> success) {
        this.success = success;
        this.failure = null;
    }

    @Override
    public Optional<ClientState> onEnter(Client client) {
        client.println("Waiting for acknowledgment...");
        return super.onEnter(client);
    }

    @Override
    public Optional<ClientState> onMessage(Client client, ServerMessage message) {
        if (message instanceof AckMessage) {
            return Optional.of(success.get());
        } else if (failure != null) {
            return Optional.of(failure.apply(message));
        } else {
            return super.onMessage(client, message);
        }
    }
}
